﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Tao.OpenGl;
using Tao.Platform.Windows;

namespace ABCof3DGL
{
    public partial class FormABCof3DGL : Form
    {
        private int viewportW = 800;
        private int viewportH = 600;
        private double aspRatio;
        private double winX = 4;
        private double winY = 4;

        public FormABCof3DGL()
        {
            InitializeComponent();
            glControl.InitializeContexts();
            this.SetBounds(0, 0, 800, 600);
            
            initGl();
            initLights();
        }
        public void initGl()
        {
            Gl.glViewport(0, 0, viewportW, viewportH);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            if (viewportW > viewportH)
            {
                aspRatio = (double)viewportW / viewportH;
                Gl.glOrtho(-winX * aspRatio, winX * aspRatio, -winY, winY, -10, 10);
            }
            else
            {
                aspRatio = (double)viewportH / viewportW;
                Gl.glOrtho(-winX, winX, -winY * aspRatio, winY * aspRatio, -10, 10);
            }

            Gl.glMatrixMode(Gl.GL_MODELVIEW);         
            Gl.glLoadIdentity();
            Glu.gluLookAt(2, 2, 2, 0, 0, 0, 0, 1, 0);
        }
        public void initLights()
        {
            float[] lightPos = { 1.5f, 2.6f, 4.0f, 1.0f };
            float[] ambi = { 0.25f, 0.2f, 0.2f, 0.0f };
            float[] diff = { 1.0f, 0.1f, 0.0f, 0.0f };
            float[] spec = { 0.8f, 0.8f, 0.8f, 0.0f };

            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, lightPos);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, ambi);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, diff);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, spec);

            Gl.glShadeModel(Gl.GL_FLAT); //glShadeModel(GL_FLAT);

            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT); // clear the screen
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);  // background is blackg

            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_LIGHT0);
        }

        private void glControl_Paint(object sender, PaintEventArgs e)
        {
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);

            Gl.glPushMatrix();
                Gl.glRotated(80, 0, 1, 0);
                Gl.glTranslated(2, 0, 0);
                Gl.glLineWidth(1.5f);
                WireSphere(1.3f, 20, 20);
                Gl.glTranslated(-4, 0, 0);
                Sphere(1.3f, 20, 20);
            Gl.glPopMatrix();

            Gl.glFlush();
            Gl.glLineWidth(1.0f);
        }

        private void FormABCof3DGL_Resize(object sender, EventArgs e)
        {
            viewportW = ClientRectangle.Width;
            viewportH = ClientRectangle.Height;

            Gl.glViewport(0, 0, viewportW, viewportH);

            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            if (viewportW < viewportH)
            {
                aspRatio = (double)viewportH / viewportW;
                Gl.glOrtho(-winX, winX, -winY * aspRatio, winY * aspRatio, -4, 4);
            }
            else
            {
                aspRatio = (double)viewportW / viewportH;
                Gl.glOrtho(-winX * aspRatio, winX *aspRatio, -winY, winY, -4, 4);
            }
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
            Glu.gluLookAt(2, 2, 2, 0, 0, 0, 0, 1, 0);
            glControl.Refresh();
            this.Refresh();
        }
        public void Sphere(double radius, int slices, int stacks)
        {

            Glu.GLUquadric quad = Glu.gluNewQuadric();
            Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
            Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
            //Glu.gluQuadricOrientation(quad, Glu.GLU_INSIDE);
            Glu.gluSphere(quad, radius, slices, stacks);
        }
        public void WireSphere(double radius, int slices, int stacks)
        {
            Glu.GLUquadric quad = Glu.gluNewQuadric();
            Glu.gluQuadricDrawStyle(quad, Glu.GLU_LINE);//GLU_POINT,GLU_FILL,GLU_SILHOUETTE
            Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
            //Glu.gluQuadricOrientation(quad, Glu.GLU_INSIDE);
            Gl.glPushMatrix();
                Gl.glRotated(120, 1, 0, 0);
                Glu.gluSphere(quad, radius, slices, stacks);
            Gl.glPopMatrix();
        }
    }
}
